/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * Component Development Kit constants
 * 
 * @author chamerling - eBM Websourcing
 * 
 */
public class Constants {

    /**
     * The component properties
     * 
     * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
     * 
     */
    public static class Component {

        public static final String JBI_XML = "jbi.xml";

        public static final String META_INF = "META-INF";

        public static final String ACKNOWLEDGEMENT_TIMEOUT = "acknowledgement-timeout";

        /**
         * The endpoint name used as the extension: auto-generated endpoint name
         * extension
         */
        public static final String AUTOGENERATED_ENDPOINT_NAME = "petals:autogenerate";

        public static final String COMPONENT_INTERCEPTORS = "component-interceptors";

        public static final long DEFAULT_ACKNOWLEDGEMENT_TIMEOUT = 5000L;

        public static final String DEFAULT_IGNORED_STATUS = "DONE_AND_ERROR_IGNORED";

        public static final int DEFAULT_POOL_SIZE = 10;

        public static final int DEFAULT_ACCEPTOR_SIZE = 5;

        public static final String EXTERNAL_LISTENER_CLASSNAME = "external-listener-class-name";

        public static final String IGNORED_STATUS = "ignored-status";

        public static final String JBI_LISTENER_CLASSNAME = "jbi-listener-class-name";

        public static final String MEP = "mep";

        public static final String OPERATION = "operation";

        public static final String PLACEHOLDER_PROPERTIES_FILE = "properties-file";

        public static final String POOL_SIZE = "pool-size";

        public static final String ACCEPTOR_SIZE = "acceptor-size";

        public static final String SU_INTERCEPTORS = "su-interceptors";

        public static final String TIMEOUT = "timeout";

        public static final String WSDL_LOCATION = "wsdl";

    }

    /**
     * The interceptor constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     * @since 3.0
     * 
     */
    public static class Interceptor {
        public static final String DELIM = ",";

        public static final String ACCEPT = "accept";

        public static final String ACCEPT_RESPONSE = "accept-response";

        public static final String SEND = "send";

        public static final String SEND_RESPONSE = "send-response";

        public static final String INTERCEPTOR_NODE = "interceptor";

        public static final String PARAMETER_NODE = "param";

        public static final String ORDER_NODE = "order";

        public static final String CLASS_ATTR = "class";

        public static final String ACTIVE_ATTR = "active";

        public static final String MODE_ATTR = "mode";

        public static final String NAME_ATTR = "name";

        public static final String VALUE_ATTR = "value";

        public enum Phase {
            SEND, SEND_RESPONSE, ACCEPT, ACCEPT_RESPONSE;
        }
    }

    /**
     * Constants for the ignored Message Exchange Status.
     * 
     * @author ofabre
     * @author gblondelle
     */
    public enum IgnoredStatusConstants {
        DONE_AND_ERROR_IGNORED("DONE_AND_ERROR_IGNORED"), DONE_IGNORED("DONE_IGNORED"),
        ERROR_IGNORED("ERROR_IGNORED"), NOTHING_IGNORED("NOTHING_IGNORED");

        private final String value;

        /**
         * Creates a new instance of {@link IgnoredStatusConstants}
         * 
         * @param value
         */
        private IgnoredStatusConstants(String value) {
            this.value = value;
        }

        /**
         * 
         * @return
         */
        public String value() {
            return value;
        }

    }

    /**
     * Constants for the JBI Message Exchange Patterns.
     * 
     * @author ofabre
     * @author gblondelle
     */
    public enum MEPConstants {
        IN_ONLY_PATTERN("http://www.w3.org/2004/08/wsdl/in-only"), IN_OPTIONAL_OUT_PATTERN(
                "http://www.w3.org/2004/08/wsdl/in-opt-out"), IN_OUT_PATTERN(
                "http://www.w3.org/2004/08/wsdl/in-out"), ROBUST_IN_ONLY_PATTERN(
                "http://www.w3.org/2004/08/wsdl/robust-in-only");

        /**
         * 
         * @param pattern
         * @return
         */
        public static MEPConstants valueOf(final URI pattern) {
            MEPConstants result = null;
            if (pattern != null) {
                for (MEPConstants mep : MEPConstants.values()) {
                    if (mep.nameSpace.equals(pattern.toString())) {
                        result = mep;
                    }
                }
            }
            return result;
        }

        private final String nameSpace;

        /**
         * Creates a new instance of {@link MEPConstants}
         * 
         * @param nameSpace
         */
        private MEPConstants(String nameSpace) {
            this.nameSpace = nameSpace;
        }

        /**
         * 
         * @return
         */
        public URI value() {
            URI mepURI = null;
            try {
                mepURI = new URI(nameSpace);
            } catch (URISyntaxException e) {
                throw new Error("Unexpected Error in JBI URI namespace syntax", e); // NOPMD
            }
            return mepURI;
        }

        /**
         * Please use this equals method instead of using :<code>
         * value().equals(mep)
         * </code>
         * which is almost 10 times slower...
         * 
         * @param mep
         * @return
         */
        public boolean equals(URI mep) {
            return toString().equals(mep.toString());
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Enum#toString()
         */
        public String toString() {
            return nameSpace;
        }
    }

    public static class MEPShortcuts {

        /**
         * InOnly Mep shortcut
         */
        public static final String IN_ONLY = "InOnly";

        /**
         * RobustInOnly Mep shortcut
         */
        public static final String ROBUST_IN_ONLY = "RobustInOnly";

        /**
         * InOut Mep shortcut
         */
        public static final String IN_OUT = "InOut";

        /**
         * InOptionalOut Mep shortcut
         */
        public static final String IN_OPTIONAL_OUT = "InOptionalOut";
    }

    public static final String MANAGED_METHOD_PREFIX = "managedMethod";

    public static final String PARAMETER_PREFIX = "component.parameter";

    private Constants() {
    }
}
