/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 OW2 Consortium, http://www.ow2.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.listener;

import java.lang.reflect.Array;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A {@link ThreadFactory} used by the JBI Workers Pool
 * {@link ThreadPoolExecutor}.
 * 
 * Created on 14 janv. 08
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since
 * 
 */
public class JBIListenerThreadFactory implements ThreadFactory {

	ThreadGroup group = null;

	private Logger logger;

	final AtomicInteger threadNumber = new AtomicInteger(1);

	/**
	 * Create a new instance of {@link JBIListenerThreadFactory}
	 * 
	 * @param componentName
	 * @param logger
	 */
	public JBIListenerThreadFactory(final String componentName,
			final Logger logger) {

		final String jbiListenersThreadGroupName = componentName
				+ " - JBI listeners";
		this.logger = logger;

		// We check if the threadGroup of the component JBI listener already
		// exists
		int nbThreadGroup = Thread.currentThread().getThreadGroup()
				.activeGroupCount();
		ThreadGroup[] threadGroups = (ThreadGroup[]) Array.newInstance(
				ThreadGroup.class, nbThreadGroup);
		nbThreadGroup = Thread.currentThread().getThreadGroup().enumerate(
				threadGroups);
		for (int i = 0; i < nbThreadGroup; i++) {
			if (jbiListenersThreadGroupName.equals(threadGroups[i].getName())) {
				this.group = threadGroups[i];
				break;
			}
		}

		if (this.group == null) {
			this.group = new ThreadGroup(Thread.currentThread()
					.getThreadGroup(), jbiListenersThreadGroupName);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.ThreadFactory#newThread(java.lang.Runnable)
	 */
	public Thread newThread(Runnable r) {
		String name = "JBI Listener thread #"
				+ Integer.toString(threadNumber.getAndIncrement());
		this.logger.log(Level.FINE, "Creating new JBI Listener Thread : "
				+ name);
		return new Thread(this.group, r, name);
	}
}
