/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.mbean;

import java.util.Iterator;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.component.framework.ComponentConfiguration;

import static org.ow2.petals.component.framework.Constants.MANAGED_METHOD_PREFIX;

/**
 * The component information MBean used to get information about the component
 * during runtime via JMX.
 * 
 * @author Christophe Hamerling (chamerling) - eBM WebSourcing
 * 
 */
public class ComponentInformationMBean implements DynamicMBean {

    private ComponentConfiguration componentConfiguration;

    private ObjectName mbeanName;

    /**
     * Creates a new instance of {@link ComponentInformationMBean}
     * 
     * @param propertiesManager
     * @param mbeanName
     */
    public ComponentInformationMBean(ComponentConfiguration propertiesManager,
            ObjectName mbeanName) {
        this.componentConfiguration = propertiesManager;
        this.mbeanName = mbeanName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#getAttribute(java.lang.String)
     */
    public Object getAttribute(String attribute) throws AttributeNotFoundException, MBeanException,
            ReflectionException {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#getAttributes(java.lang.String[])
     */
    public synchronized AttributeList getAttributes(String[] attributes) {
        AttributeList list = new AttributeList();
        for (String name : attributes) {
            String value = this.componentConfiguration.getProperties().getProperty(name);
            if (value != null
                    && !value.startsWith(MANAGED_METHOD_PREFIX)) {
                list.add(new Attribute(name, value));
            }
        }
        return list;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#getMBeanInfo()
     */
    public MBeanInfo getMBeanInfo() {
        // get the attributes
        MBeanAttributeInfo[] attrs = getAttributesInfos();

        // create MBeanInfo with no constructors and no notifications
        return new MBeanInfo(this.getClass().getName(), "Component Information MBean", attrs, null,
                null, null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#invoke(java.lang.String,
     *      java.lang.Object[], java.lang.String[])
     */
    public Object invoke(String actionName, Object[] params, String[] signature)
            throws MBeanException, ReflectionException {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#setAttribute(javax.management.Attribute)
     */
    public void setAttribute(Attribute attribute) throws AttributeNotFoundException,
            InvalidAttributeValueException, MBeanException, ReflectionException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.management.DynamicMBean#setAttributes(javax.management.AttributeList)
     */
    public AttributeList setAttributes(AttributeList attributes) {
        return null;
    }

    private MBeanAttributeInfo[] getAttributesInfos() {
        // get the attributes names as sorted strings
        Set<Object> attributeKeys = componentConfiguration.getProperties().keySet();

        // build the MBeanAttributeInfo
        MBeanAttributeInfo[] attrs = new MBeanAttributeInfo[attributeKeys.size()];
        Iterator<Object> it = attributeKeys.iterator();
        for (int i = 0; i < attrs.length; i++) {
            String name = (String)it.next();
            attrs[i] = new MBeanAttributeInfo(name, "java.lang.String", "Property " + name, true,
                    false, false);
        }
        return attrs;
    }

    /**
     * 
     * @return
     */
    public ObjectName getMbeanName() {
        return mbeanName;
    }
}
