/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.mbean;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanNotificationInfo;
import javax.management.NotificationEmitter;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.interceptor.Interceptor;
import org.ow2.petals.component.framework.interceptor.MessageInterceptorManager;
import org.ow2.petals.component.framework.interceptor.description.InterceptorBuilder;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.jbi.descriptor.ServiceUnitExtensibleElement;

/**
 * Add interceptors at runtime
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 3.0
 * 
 */
public class InterceptorManager implements InterceptorManagerMBean,
		NotificationEmitter {

	private AbstractComponent component;

	private Logger logger;

	private ObjectName mBeanName;

	/**
	 * 
	 * @param component
	 */
	public InterceptorManager(AbstractComponent component) {
		this.component = component;
		this.logger = component.getLogger();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.InterceptorManagerMBean#engageInterceptorToComponent(java.lang.String,
	 *      java.lang.String)
	 */
	public void addInterceptor(String className, String interceptorName)
			throws MBeanException {
		this.logger.log(Level.INFO, "Add interceptor " + interceptorName);

		if (this.component.getInterceptor(interceptorName) != null) {
			throw new MBeanException(new Exception(
					"This interceptor is already registered"));
		}

		InterceptorBuilder builder = InterceptorBuilder.getInstance();
		Interceptor itc = null;
		try {
			itc = builder.loadInstance(className);
			itc.init(this.component, interceptorName, false,
					new HashMap<String, String>(), this.logger);
		} catch (PEtALSCDKException e) {
			throw new MBeanException(e);
		}

		Map<String, Interceptor> interceptors = this.component
				.getInterceptors();
		interceptors.put(interceptorName, itc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.InterceptorManagerMBean#startInterceptor(java.lang.String)
	 */
	public boolean startInterceptor(String interceptorName)
			throws MBeanException {
		this.logger.log(Level.INFO, "Start interceptor " + interceptorName);

		boolean result = false;

		Interceptor itc = this.component.getInterceptor(interceptorName);
		if (itc == null) {
			throw new MBeanException(new Exception(
					"No interceptor found with name " + interceptorName));
		}

		if (!itc.isActive()) {
			itc.setActive(true);
			result = true;
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.InterceptorManagerMBean#stopInterceptor(java.lang.String)
	 */
	public boolean stopInterceptor(String interceptorName)
			throws MBeanException {
		this.logger.log(Level.INFO, "Stop interceptor " + interceptorName);

		boolean result = false;

		Interceptor itc = this.component.getInterceptor(interceptorName);
		if (itc == null) {
			throw new MBeanException(new Exception(
					"No interceptor found with name " + interceptorName));
		}

		if (itc.isActive()) {
			itc.setActive(false);
			result = true;
		}
		return result;
	}

	/**
	 * 
	 * @param interceptorName
	 */
	public void engageInterceptor(String interceptorName) throws MBeanException {
		Interceptor interceptor = this.component
				.getInterceptor(interceptorName);
		if (interceptor == null) {
			throw new MBeanException(new Exception());
		}

		ServiceUnitManager suManager = (ServiceUnitManager) this.component
				.getServiceUnitManager();

		// TODO refactor the SU manager
		Map<String, ServiceUnitDataHandler> suDatahandlers = suManager
				.getServiceUnitDataHandlers();
		Iterator<String> iter = suDatahandlers.keySet().iterator();
		while (iter.hasNext()) {
			String key = iter.next();
			ServiceUnitDataHandler suDataHandler = suDatahandlers.get(key);
			Map<ServiceUnitExtensibleElement, MessageInterceptorManager> managers = suDataHandler
					.getMessageInterceptorManager();

			Iterator<ServiceUnitExtensibleElement> iter2 = managers.keySet()
					.iterator();

			while (iter2.hasNext()) {
				ServiceUnitExtensibleElement serviceUnitExtensibleElement = iter2
						.next();
				MessageInterceptorManager manager = managers
						.get(serviceUnitExtensibleElement);
				manager.addGlobalInterceptor(interceptor);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.management.NotificationEmitter#removeNotificationListener(javax.management.NotificationListener,
	 *      javax.management.NotificationFilter, java.lang.Object)
	 */
	public void removeNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback)
			throws ListenerNotFoundException {
	}

	/**
	 * 
	 * @param mbeanName
	 */
	public void setMBeanName(ObjectName mbeanName) {
		this.mBeanName = mbeanName;
	}

	public void addNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback)
			throws IllegalArgumentException {
	}

	public MBeanNotificationInfo[] getNotificationInfo() {
		return null;
	}

	public void removeNotificationListener(NotificationListener listener)
			throws ListenerNotFoundException {
	}

	/**
	 * 
	 * @return
	 */
	public ObjectName getMBeanName() {
		return this.mBeanName;
	}

}
