/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.mbean;

import java.util.concurrent.atomic.AtomicLong;

import javax.jbi.JBIException;
import javax.management.AttributeChangeNotification;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationEmitter;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.TabularData;

import org.ow2.petals.component.framework.ComponentConfiguration;

/**
 * A runtime configuration MBean which sends notifications to subscribers. It is
 * used to change configuration values at runtime.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 3.0
 * 
 */
public class RuntimeConfiguration implements RuntimeConfigurationMBean,
		NotificationEmitter {

	public static final String ACCEPTOR_THREADS = "AcceptorThreads";

	public static final String WORKER_TREADS = "WorkerThreads";

	public static final String ENVIRONMENT_VARIABLES = "EnvironmentVariables";

	private NotificationBroadcasterSupport broadcasterSupport = new NotificationBroadcasterSupport();

	long MIN_THREADS = 1;

	long MAX_THREADS = 255;

	private ComponentConfiguration componentConfiguration;

	private ObjectName mbeanName;

	private AtomicLong seqNumber = new AtomicLong(0);

	/**
	 * 
	 * @param workspaceRoot
	 * @param properties
	 * @throws JBIException
	 */
	public RuntimeConfiguration(ComponentConfiguration componentConfiguration) {
		this.componentConfiguration = componentConfiguration;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfigurationMBean#getEnvironmentVariables()
	 */
	public TabularData getEnvironmentVariables() throws OpenDataException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfigurationMBean#getThreads()
	 */
	public Integer getAcceptorThreads() {
		return Integer.valueOf(this.componentConfiguration.getAcceptorSize());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfigurationMBean#setThreads(java.lang.Integer)
	 */
	public void setAcceptorThreads(Integer i) throws InvalidAttributeValueException,
			MBeanException {
		Integer newVal = null;
		try {
			newVal = i;
		} catch (Exception ex) {
			throw new InvalidAttributeValueException("Invalid argument");
		}
		if ((newVal.intValue() < this.MIN_THREADS)
				|| (newVal.intValue() > this.MAX_THREADS)) {
			throw new InvalidAttributeValueException("Bad value");
		}

		String oldValue = this.getAcceptorThreads().toString();

		this.componentConfiguration.setAcceptorSize(i);
		// persistConfiguration();
		this.saveAndNotifyListners("Attribute changed", ACCEPTOR_THREADS,
				oldValue, i.toString());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfigurationMBean#getThreads()
	 */
	public Integer getWorkerThreads() {
		return Integer.valueOf(this.componentConfiguration.getPoolSize());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfigurationMBean#setThreads(java.lang.Integer)
	 */
	public void setWorkerThreads(Integer i) throws InvalidAttributeValueException,
			MBeanException {
		Integer newVal = null;
		try {
			newVal = i;
		} catch (Exception ex) {
			throw new InvalidAttributeValueException("Invalid argument");
		}
		if ((newVal.intValue() < this.MIN_THREADS)
				|| (newVal.intValue() > this.MAX_THREADS)) {
			throw new InvalidAttributeValueException("Bad value");
		}

		String oldValue = this.getWorkerThreads().toString();

		this.componentConfiguration.setPoolSize(i);
		// persistConfiguration();
		this.saveAndNotifyListners("Attribute changed", WORKER_TREADS,
				oldValue, i.toString());
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.management.NotificationBroadcaster#getNotificationInfo()
	 */
	public MBeanNotificationInfo[] getNotificationInfo() {
		return new MBeanNotificationInfo[] { new MBeanNotificationInfo(
				new String[] { AttributeChangeNotification.ATTRIBUTE_CHANGE },
				AttributeChangeNotification.class.getName(),
				"Change runtime configuration attribute") };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.management.NotificationBroadcaster#addNotificationListener(javax.management.NotificationListener,
	 *      javax.management.NotificationFilter, java.lang.Object)
	 */
	public void addNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback) {
		this.broadcasterSupport.addNotificationListener(listener, filter,
				handback);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.management.NotificationBroadcaster#removeNotificationListener(javax.management.NotificationListener)
	 */
	public void removeNotificationListener(NotificationListener listener)
			throws ListenerNotFoundException {
		this.broadcasterSupport.removeNotificationListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.management.NotificationEmitter#removeNotificationListener(javax.management.NotificationListener,
	 *      javax.management.NotificationFilter, java.lang.Object)
	 */
	public void removeNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback)
			throws ListenerNotFoundException {
		this.broadcasterSupport.removeNotificationListener(listener, filter,
				handback);
	}

	/**
	 * FIXME : New value to be saved in the properties file.
	 * 
	 * @param msg
	 * @param attrName
	 * @param oldVal
	 * @param newVal
	 */
	private void saveAndNotifyListners(String msg, String attrName,
			String oldVal, String newVal) {
		Notification notif = new AttributeChangeNotification(this, seqNumber
				.getAndIncrement(), System.currentTimeMillis(), msg, attrName,
				String.class.getName(), oldVal, newVal);
		this.broadcasterSupport.sendNotification(notif);
	}

	/**
	 * Get the MBean Name
	 * 
	 * @return
	 */
	public ObjectName getMBeanName() {
		return this.mbeanName;
	}
	
	/**
	 * 
	 * @param mbeanName
	 */
	public void setMBeanName(ObjectName mbeanName) {
		this.mbeanName = mbeanName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.component.framework.mbean.RuntimeConfiguration#getComponentConfiguration()
	 */
	public ComponentConfiguration getComponentConfiguration() {
		return componentConfiguration;
	}
}
