/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.su;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.component.framework.interceptor.MessageInterceptorManager;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.Provides;
import org.ow2.petals.jbi.descriptor.ServiceUnitExtensibleElement;
import org.w3c.dom.Document;

/**
 * Handles useful information about a Service Unit like :<br/> - the service
 * unit description (jbi descriptor)<br/> - links between Provides nodes and
 * activated endpoints<br/> - links between activated endpoints and
 * corresponding service description<br/> - links between Consumes nodes and
 * corresponding addresses<br/> - the installation root<br/> - the service
 * unit name - the message interceptor manager<br/> A SUDataHandler is
 * available for each SU.
 * 
 * @author ofabre
 * 
 */
public class ServiceUnitDataHandler {

    private JBIDescriptor descriptor;

    private Map<ServiceEndpoint, Provides> epJBIDesc;

    private Map<ServiceEndpoint, Document> epServiceDesc;

    private List<Consumes> consumesList;

    /**
     * Map of message interceptor managers. Each provide/consume element has its
     * own preloaded manager.
     */
    private Map<ServiceUnitExtensibleElement, MessageInterceptorManager> messageInterceptorManager;

    private String installRoot;

    private String name;

    /**
     * 
     * Creates a new instance of {@link ServiceUnitDataHandler}
     * 
     */
    public ServiceUnitDataHandler() {
        super();
        this.consumesList = new ArrayList<Consumes>();
        this.epServiceDesc = new HashMap<ServiceEndpoint, Document>();
        this.epJBIDesc = new HashMap<ServiceEndpoint, Provides>();
        this.messageInterceptorManager = new HashMap<ServiceUnitExtensibleElement, MessageInterceptorManager>();
    }

    /**
     * 
     * @param endpoint
     * @param description
     */
    public void addEndpointDescription(final ServiceEndpoint endpoint, final Document description) {
        this.epServiceDesc.put(endpoint, description);
    }

    /**
     * Return wsdl description of the given endpoint
     * 
     * @param endpoint
     */
    public Document getEndpointDescription(final ServiceEndpoint endpoint) {
        return this.epServiceDesc.get(endpoint);
    }

    /**
     * 
     * @param endpoint
     * @param provides
     */
    public void addJBIDescription(final ServiceEndpoint endpoint, final Provides provides) {
        this.epJBIDesc.put(endpoint, provides);
    }

    /**
     * 
     * @return
     */
    public JBIDescriptor getDescriptor() {
        return this.descriptor;
    }

    /**
     * 
     * @return
     */
    public Map<ServiceEndpoint, Provides> getEpJBIDesc() {
        return this.epJBIDesc;
    }

    /**
     * 
     * @return
     */
    public Map<ServiceEndpoint, Document> getEpServiceDesc() {
        return this.epServiceDesc;
    }

    /**
     * 
     * @return
     */
    public String getInstallRoot() {
        return this.installRoot;
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return this.name;
    }

    /**
     * 
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * 
     * @param endpoint
     */
    public void removeEndpoint(final ServiceEndpoint endpoint) {
        this.epServiceDesc.remove(endpoint);
        this.epJBIDesc.remove(endpoint);
    }

    /**
     * 
     * @param descriptor
     */
    public void setDescriptor(final JBIDescriptor descriptor) {
        this.descriptor = descriptor;
    }

    /**
     * 
     * @param epJBIDesc
     */
    public void setEpJBIDesc(final Map<ServiceEndpoint, Provides> epJBIDesc) {
        this.epJBIDesc = epJBIDesc;
    }

    /**
     * 
     * @param epServiceDesc
     */
    public void setEpServiceDesc(final Map<ServiceEndpoint, Document> epServiceDesc) {
        this.epServiceDesc = epServiceDesc;
    }

    /**
     * 
     * @param installRoot
     */
    public void setInstallRoot(final String installRoot) {
        this.installRoot = installRoot;
    }

    /**
     * 
     * @return
     */
    public List<Consumes> getConsumesList() {
        return this.consumesList;
    }

    /**
     * 
     * @param consumesList
     */
    public void setConsumesList(List<Consumes> consumesList) {
        this.consumesList = consumesList;
    }

    /**
     * 
     * @param consumes
     */
    public void addConsumes(Consumes consumes) {
        this.consumesList.add(consumes);
    }

    /**
     * Add an interceptor manager.
     * 
     * @param extensible
     * @param manager
     */
    public void addMessageInterceptorManager(ServiceUnitExtensibleElement extensible,
            MessageInterceptorManager manager) {
        this.messageInterceptorManager.put(extensible, manager);
    }

    /**
     * Get the manager for the given provide/consume element
     * 
     * @param extensible
     * @return
     */
    public MessageInterceptorManager getMessageInterceptorManager(
            ServiceUnitExtensibleElement extensible) {
        return this.messageInterceptorManager.get(extensible);
    }

    /**
     * @return the messageInterceptorManager
     */
    public Map<ServiceUnitExtensibleElement, MessageInterceptorManager> getMessageInterceptorManager() {
        return this.messageInterceptorManager;
    }

}
